package model;

import java.util.Iterator;

/**
 * una implementazione di Listaordinata
 * @author giovanni.ragno
 */
public class Elenco<T extends Comparable<T>> implements ListaOrdinata<T>, Iterable<T> {
  Nodo<T> testa;
  public Elenco() {
    testa=null;
  }

  @Override
  public void inserisci(T dato) {
    if (testa==null || testa.getDato().compareTo(dato) >= 0){
      testa=new Nodo<T>(dato,testa);
    }
    else {
      Nodo<T> cursore=testa;
      while (cursore.getProssimo()!=null && cursore.getProssimo().getDato().compareTo(dato)<0) {
        cursore=cursore.getProssimo();
      }
      cursore.setProssimo(new Nodo<T>(dato,cursore.getProssimo()));
    }
  }

  @Override
  public int lunghezza() {
    int l=0;
    Nodo<T> cursore=testa;
    while (cursore!=null){
      cursore=cursore.getProssimo();
      l++;
    }
    return l;
  }

  @Override
  public int posto(T dato) {
    if (testa!=null){
      Nodo<T> cursore=testa;
      int pos=0;
      while (cursore.getProssimo()!=null && cursore.getDato().compareTo(dato)<0){
        cursore=cursore.getProssimo();
        pos++;
      }
      if (cursore.getDato().compareTo(dato)==0){
        return pos;
      }
    }
    return -1;
  }

  @Override
  public void rimuovi(T dato) {
    if (testa!=null){
      if (testa.getDato().compareTo(dato)==0){
        testa=testa.getProssimo();
      }
      else {
        Nodo<T> cursore=getNodoPrecedente(dato);
        if (cursore!=null && cursore.getProssimo()!= null && cursore.getProssimo().getDato().compareTo(dato)==0){
          cursore.setProssimo(cursore.getProssimo().getProssimo());
        }
      }
    }
  }
  public void rimuoviO(T dato) {
    if (testa!=null){
      if (testa.getDato().compareTo(dato)==0){
        testa=testa.getProssimo();
      }
      else {
        Nodo<T> cursore=testa;
        while (cursore.getProssimo()!=null && cursore.getProssimo().getDato().compareTo(dato)<0){
          cursore=cursore.getProssimo();
        }
        if (cursore.getProssimo()!= null && cursore.getProssimo().getDato().compareTo(dato)==0){
          cursore.setProssimo(cursore.getProssimo().getProssimo());
        }
      }
    }
  }
  private Nodo<T> getNodoPrecedente(T dato){
    Nodo<T> cursore=testa;
    if (cursore!=null && cursore.getDato().compareTo(dato)<0){
      while (cursore.getProssimo()!=null && cursore.getProssimo().getDato().compareTo(dato)<0){
        cursore=cursore.getProssimo();
      }
    }
    else {
      cursore=null;
    }
    return cursore;
  }

  @Override
  public void svuota() {
    testa=null;
  }

  @Override
  public String toString() {
    return "Elenco{" + "testa=" + testa + '}';
  }

  @Override
  public Iterator<T> iterator() {
    return new Iteratore<T>();
  }
  
  
  private class Nodo<T>{
    private T dato;
    private Nodo<T> prossimo;

    public Nodo(T dato, Nodo<T> prossimo) {
      this.dato = dato;
      this.prossimo = prossimo;
    }

    public T getDato() {
      return dato;
    }

    public void setDato(T dato) {
      this.dato = dato;
    }

    public Nodo<T> getProssimo() {
      return prossimo;
    }

    public void setProssimo(Nodo<T> prossimo) {
      this.prossimo = prossimo;
    }

    @Override
    public String toString() {
      return "Nodo{" + "dato=" + dato + ", prossimo=" + prossimo + '}';
    }
    
  }
  
  private class Iteratore<T> implements Iterator<T> {
    Nodo<T> cursore;

    public Iteratore() {
      cursore=(Nodo<T>) testa;
    }

    
    @Override
    public boolean hasNext() {
      return cursore != null;
    }

    @Override
    public T next() {
      if (cursore==null){
        throw new RuntimeException("Impossibile avanzare");
      }
      T dato=cursore.getDato();
      cursore=cursore.getProssimo();
      return dato;
    }

    @Override
    public void remove() {
      throw new UnsupportedOperationException("Not supported yet.");
    }
    
  }
}
