package model;

/**
 *
 * @author giovanni.ragno
 */
public class ListaConcatenata<T> implements Lista<T> {
  protected Nodo<T> testa;

  public ListaConcatenata() {
    testa=null;
  }

  @Override
  public void appendiInCoda(T dato) {
//    inserisciInPosizione(dato, length());//bello, ma poco efficiente
    if (testa==null){
      //caso particolare
      inserisciInTesta(dato);
    }
    else {
      Nodo<T> cursore=testa; //so per certo che non  null
      //porta cursore all'ultimo nodo
      while (cursore.getProssimo()!=null){
        cursore=cursore.getProssimo();
      }
      //appendi dato
      cursore.setProssimo(new Nodo<T>(dato,null));
    }
  }

  @Override
  public T getDatoInCoda() {
//      return getNodoInPosizione(length()-1).getDato();//bello, ma poco efficiente
    if (testa==null){
      //caso particolare
      throw new RuntimeException("Impossibile accedere a dato su lista vuota");
    }
    else {
      Nodo<T> cursore=testa; //so per certo che non  null
      //porta cursore all'ultimo nodo
      while (cursore.getProssimo()!=null){
        cursore=cursore.getProssimo();
      }
      //restituisci dato
      return cursore.getDato();
    }
  }

  @Override
  public T getDatoInPosizione(int pos) {
    return getNodoInPosizione(pos).getDato();
  }

  @Override
  public T getDatoInTesta() {
    if (testa==null){
      throw new RuntimeException("Impossibile accedere a dato su lista vuota");
    }
    return testa.getDato();
  }

  @Override
  public void inserisciInPosizione(T dato, int pos) {
    if (pos==0){
      inserisciInTesta(dato);
    }
    else {
      Nodo<T> n = getNodoInPosizione(pos-1);
      Nodo<T> nuovo = new Nodo<T>(dato,n.getProssimo());
      n.setProssimo(nuovo);
    }
  }

  @Override
  public void inserisciInTesta(T dato) {
    Nodo<T> nuovo = new Nodo<T>(dato,testa);
    testa=nuovo;
  }

  @Override
  public int length() {
    return length(testa);
  }

  private int length(Nodo<T> testa) {
    if (testa==null){
      return 0;
    }
    return 1+length(testa.getProssimo());
  }

  @Override
  public void rimuoviDaCoda() {
//    rimuoviDaPosizione(length()-1);//bello, ma poco efficiente
    if (testa==null){
      //caso particolare
      throw new RuntimeException("Impossibile eliminare dato alla posizione richiesta");
    }
    else {
      if (testa.getProssimo()==null){
        // caso particolare, ho solo un nodo
        testa=null;
      }
      else {
        Nodo<T> cursore=testa; //so per certo che non  null testa e testa.getProssimo()
        //porta cursore al penultimo nodo
        while (cursore.getProssimo().getProssimo()!=null){
          cursore=cursore.getProssimo();
        }
        //elimina il prossimo, che  l'ultimo nodo
        cursore.setProssimo(null);
      }
    }
  }

  @Override
  public void rimuoviDaPosizione(int pos) {
    if (pos==0){
      rimuoviDaTesta();
    }
    else {
      Nodo n = getNodoInPosizione(pos-1);
      if (n.getProssimo()==null){
        throw new RuntimeException("Impossibile eliminare dato alla posizione richiesta");
      }        
      n.setProssimo(n.getProssimo().getProssimo());
    }
  }

  @Override
  public void rimuoviDaTesta() {
    if (testa==null){
      throw new RuntimeException("Impossibile eliminare dato su lista vuota");
    }
    testa=testa.getProssimo();
  }

  @Override
  public void setDatoInCoda(T dato) {
//    getNodoInPosizione(length()-1).setDato(dato);//bello, ma poco efficiente
    if (testa==null){
      //caso particolare
      throw new RuntimeException("Impossibile modificare dato su lista vuota");
    }
    else {
      Nodo<T> cursore=testa; //so per certo che non  null
      //porta cursore all'ultimo nodo
      while (cursore.getProssimo()!=null){
        cursore=cursore.getProssimo();
      }
      //set dato
      cursore.setDato(dato);
    }
  }

  @Override
  public void setDatoInPosizione(T dato, int pos) {
    getNodoInPosizione(pos).setDato(dato);
  }

  @Override
  public void setDatoInTesta(T dato) {
    if (testa==null){
      throw new RuntimeException("Impossibile modificare dato su lista vuota");
    }
    testa.setDato(dato);
  }

  @Override
  public void svuota() {
    testa=null;//ci si appella al garbage collector... in altri linguaggi va liberato nodo per nodo
  }

  @Override
  public String toString() {
    return "ListaConcatenata{" + "testa=" + testa + '}';
  }

  private Nodo<T> getNodoInPosizione(int pos){
    if (pos<0){
      throw new RuntimeException("Posizione deve essere >= 0");
    }
    Nodo<T> cursore = testa;
    for (int i = 0; i < pos; i++) {
      if(cursore==null){
        throw new RuntimeException("Impossibile raggiungere nodo nella posizione richiesta");
      }
      cursore=cursore.getProssimo();            
    }
    if(cursore==null){
      throw new RuntimeException("Impossibile accedere a dato nella posizione richiesta");
    }
    return cursore;
  }

  private class Nodo<T> {
    T dato;
    Nodo<T> prossimo;

    public Nodo() {
      dato=null;
      prossimo=null;
    }

    public Nodo(T dato, Nodo<T> prossimo) {
      this.dato = dato;
      this.prossimo = prossimo;
    }

    public T getDato() {
      return dato;
    }

    public void setDato(T dato) {
      this.dato = dato;
    }

    public Nodo<T> getProssimo() {
      return prossimo;
    }

    public void setProssimo(Nodo<T> prossimo) {
      this.prossimo = prossimo;
    }

    @Override
    public String toString() {
      return "Nodo{" + "dato=" + dato + ", prossimo=" + prossimo + '}';
    }
  }
}
